pub struct MemoryMappedFile<LOCK: MMFLock> { /* private fields */ }Expand description
A simple struct wrapping a Memory Mapped File.
It contains all the data required to create and keep alive a [HANDLE] to a Memory Mapped File. The [HANDLE] is
required to create a [MEMORY_MAPPED_VIEW_ADDRESS] to read and write data from. In order to expose reading and
writing functionality in a safe manner, these are wrapped in a safe API that prefers more short blocks of unsafe
over a larger block that also does safe operations.
Once this reaches a stable-ish state, an unsafe function feature will be added to get access to the raw handle and
map view.
Supports both x86 and AMD64 by leveraging usize, to allow target-sized ints to be used everywhere.
Implementations
Sourceimpl<LOCK: MMFLock> MemoryMappedFile<LOCK>
impl<LOCK: MMFLock> MemoryMappedFile<LOCK>
Sourcepub fn new(
size: NonZeroUsize,
name: impl Into<ztr64>,
namespace: Namespace,
large_pages: Option<bool>,
) -> MMFResult<Self>
pub fn new( size: NonZeroUsize, name: impl Into<ztr64>, namespace: Namespace, large_pages: Option<bool>, ) -> MMFResult<Self>
Attempt to create a new Memory Mapped File. Or fail graciously if we can’t.
The size will be automatically divided into the upper and lower halves, as the function to allocate this memory requires them to be split. The name of the file should be any one of:
- Just a filename, if the namespace is either
Namespace::GLOBALorNamespace::LOCAL - A namespaced filename if using
Namespace::CUSTOMand you know what you’re doing - Just a filename if using
Namespace::CUSTOMand you don’t need other processes to access it.
Violating these constraints should result in a local namespace, but no guarantees are given and if it leads to UB, the heat death of the universe, panics or errors or a change in the answer to a value other than 42. you’re on your own.
The size MUST be a non-zero value; allocating zero bytes errors on the OS end of things. Allocating too much will make a part of the file inaccessible to other code trying to read it from a 32-bit process. The total size allocated will be 4 bytes larger than the specified size, but only after checking the input size is non-zero.
Windows will, by default, overcommit filemappings to be multiples of the used page size. It does this silently for normal pages, but requires it for large pages. As of 0.5.0 (when large page support was added), this crate will mimic this behavior and provide users with the next (checked to not overflow) integer multiple of the selected page size, minus the 4 bytes for the lock. Large page usage requiring mapping them in full is a technical limitation.
To support use cases where permissions for large pages might not be available, it is possible to explicitly prevent their use. As such, large page usage is enabled if the system supports it, and
large pagesis explicitly true- the requested size + 4 is at least as big as a large page, and
large_pagesisn’tSome(false)
In most cases large pages will be 2MB or 4MB. If your architecture doesn’t support large pages, it makes a best effort to get an exact multiple of your page size close to, but larger than, the requested size + 4. The exact sizes for (large) pages in Windows can be found in this Old New Thing from 2021.
Sourcepub fn open(
size: NonZeroUsize,
name: impl Into<ztr64>,
namespace: Namespace,
readonly: bool,
large_pages: Option<bool>,
) -> MMFResult<Self>
pub fn open( size: NonZeroUsize, name: impl Into<ztr64>, namespace: Namespace, readonly: bool, large_pages: Option<bool>, ) -> MMFResult<Self>
Open an existing MMF, if it exists.
Defaults to read and write permissions, use the exposed wrappers to open R or RW I have no idea what happens if you call this on a fake name. Code responsibly. In all reality though, it should return an error that you can handle.
Sourcepub fn open_read(
size: NonZeroUsize,
name: &str,
namespace: Namespace,
large_pages: Option<bool>,
) -> MMFResult<Self>
pub fn open_read( size: NonZeroUsize, name: &str, namespace: Namespace, large_pages: Option<bool>, ) -> MMFResult<Self>
Open an MMF for reading
Wrapper around open that always passes true
Sourcepub fn open_write(
size: NonZeroUsize,
name: &str,
namespace: Namespace,
large_pages: Option<bool>,
) -> MMFResult<Self>
pub fn open_write( size: NonZeroUsize, name: &str, namespace: Namespace, large_pages: Option<bool>, ) -> MMFResult<Self>
Open an MMF for reading and writing
Wrapper around open that always passes false
Sourcepub fn is_writable(&self) -> bool
pub fn is_writable(&self) -> bool
Check if this MMF can be written to
Sourcepub fn is_readable(&self) -> bool
pub fn is_readable(&self) -> bool
Check if this MMF can be read from
Sourcepub fn namespace(&self) -> String
pub fn namespace(&self) -> String
Get the namespace of the file, if any. If an empty string is returned, it’s Local.
Sourcepub fn filename(&self) -> String
pub fn filename(&self) -> String
Return the filename the MMF is bound to, which is only the whole name if no namespace is provided.